<?php

namespace Litecms\Homepage\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Homepage\Forms\Carousel as CarouselForm;
use Litecms\Homepage\Http\Requests\CarouselRequest;
use Litecms\Homepage\Interfaces\CarouselRepositoryInterface;
use Litecms\Homepage\Repositories\Eloquent\Filters\CarouselResourceFilter;
use Litecms\Homepage\Repositories\Eloquent\Presenters\CarouselListPresenter;

/**
 * Resource controller class for carousel.
 */
class CarouselResourceController extends BaseController
{

    /**
     * Initialize carousel resource controller.
     *
     *
     * @return null
     */
    public function __construct(CarouselRepositoryInterface $carousel)
    {
        parent::__construct();
        $this->form = CarouselForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.homepage.modules'), 'homepage', guard_url('homepage'));
        $this->repository = $carousel;
    }

    /**
     * Display a list of carousel.
     *
     * @return Response
     */
    public function index(CarouselRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(CarouselResourceFilter::class)
            ->setPresenter(CarouselListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('homepage::carousel.names'))
            ->view('homepage::carousel.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display carousel.
     *
     * @param Request $request
     * @param Model   $carousel
     *
     * @return Response
     */
    public function show(CarouselRequest $request, CarouselRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('homepage::carousel.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('homepage::carousel.show')
            ->output();
    }

    /**
     * Show the form for creating a new carousel.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(CarouselRequest $request, CarouselRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('homepage::carousel.name'))
            ->view('homepage::carousel.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new carousel.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CarouselRequest $request, CarouselRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('homepage::carousel.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('homepage/carousel/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/homepage/carousel'))
                ->redirect();
        }

    }

    /**
     * Show carousel for editing.
     *
     * @param Request $request
     * @param Model   $carousel
     *
     * @return Response
     */
    public function edit(CarouselRequest $request, CarouselRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('homepage::carousel.name'))
            ->view('homepage::carousel.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the carousel.
     *
     * @param Request $request
     * @param Model   $carousel
     *
     * @return Response
     */
    public function update(CarouselRequest $request, CarouselRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('homepage::carousel.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('homepage/carousel/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('homepage/carousel/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the carousel.
     *
     * @param Model   $carousel
     *
     * @return Response
     */
    public function destroy(CarouselRequest $request, CarouselRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('homepage::carousel.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('homepage/carousel/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('homepage/carousel/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
