<?php

namespace Litecms\Homepage\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Homepage\Forms\Catalog as CatalogForm;
use Litecms\Homepage\Http\Requests\CatalogRequest;
use Litecms\Homepage\Interfaces\CatalogRepositoryInterface;
use Litecms\Homepage\Repositories\Eloquent\Filters\CatalogResourceFilter;
use Litecms\Homepage\Repositories\Eloquent\Presenters\CatalogListPresenter;

/**
 * Resource controller class for catalog.
 */
class CatalogResourceController extends BaseController
{

    /**
     * Initialize catalog resource controller.
     *
     *
     * @return null
     */
    public function __construct(CatalogRepositoryInterface $catalog)
    {
        parent::__construct();
        $this->form = CatalogForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.homepage.modules'), 'homepage', guard_url('homepage'));
        $this->repository = $catalog;
    }

    /**
     * Display a list of catalog.
     *
     * @return Response
     */
    public function index(CatalogRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(CatalogResourceFilter::class)
            ->setPresenter(CatalogListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('homepage::catalog.names'))
            ->view('homepage::catalog.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display catalog.
     *
     * @param Request $request
     * @param Model   $catalog
     *
     * @return Response
     */
    public function show(CatalogRequest $request, CatalogRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('homepage::catalog.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('homepage::catalog.show')
            ->output();
    }

    /**
     * Show the form for creating a new catalog.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(CatalogRequest $request, CatalogRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('homepage::catalog.name'))
            ->view('homepage::catalog.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new catalog.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CatalogRequest $request, CatalogRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('homepage::catalog.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('homepage/catalog/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/homepage/catalog'))
                ->redirect();
        }

    }

    /**
     * Show catalog for editing.
     *
     * @param Request $request
     * @param Model   $catalog
     *
     * @return Response
     */
    public function edit(CatalogRequest $request, CatalogRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('homepage::catalog.name'))
            ->view('homepage::catalog.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the catalog.
     *
     * @param Request $request
     * @param Model   $catalog
     *
     * @return Response
     */
    public function update(CatalogRequest $request, CatalogRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('homepage::catalog.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('homepage/catalog/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('homepage/catalog/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the catalog.
     *
     * @param Model   $catalog
     *
     * @return Response
     */
    public function destroy(CatalogRequest $request, CatalogRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('homepage::catalog.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('homepage/catalog/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('homepage/catalog/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
